import os
import subprocess

# TODO: more tests


def test_entry_point() -> None:
    proc = subprocess.run(
        ['pyright-langserver'],
        check=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    )
    assert proc.returncode == 1
    output = proc.stdout.decode('utf-8')
    assert 'Connection input stream is not set' in output


def test_user_special_characters() -> None:
    proc = subprocess.run(
        ['pyright-langserver'],
        check=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        env={
            **os.environ,
            'LOGNAME': 'alice@example.com',
        },
    )
    assert proc.returncode == 1
    output = proc.stdout.decode('utf-8')
    assert 'Connection input stream is not set' in output



from __future__ import annotations

import pytest

from pyright._utils import _should_warn_version


@pytest.fixture(autouse=True)
def mock_setup(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.delenv('PYRIGHT_PYTHON_IGNORE_WARNINGS', raising=False)
    monkeypatch.delenv('PYRIGHT_PYTHON_PYLANCE_VERSION', raising=False)

    def _get_latest_version() -> None:
        raise RuntimeError('this should be overriden in tests')

    monkeypatch.setattr('pyright._utils.get_latest_version', _get_latest_version)


def test_quiet_flag() -> None:
    assert not _should_warn_version(args=(), quiet=True)


def test_outputjson_flag() -> None:
    assert not _should_warn_version(args=('--outputjson',), quiet=None)


@pytest.mark.parametrize(
    'env_value, expected',
    [
        ('1', False),
        ('true', False),
        ('0', True),
        ('false', True),
    ],
)
def test_ignore_warnings_env_var(monkeypatch: pytest.MonkeyPatch, env_value: str, expected: bool) -> None:
    monkeypatch.setenv('PYRIGHT_PYTHON_IGNORE_WARNINGS', env_value)

    if expected is True:
        monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.1')
        monkeypatch.setattr('pyright._utils.__version__', '1.0.0')
    else:
        monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.0')
        monkeypatch.setattr('pyright._utils.__version__', '1.0.0')

    assert _should_warn_version(args=(), quiet=None) == expected


def test_pylance_version_env_var(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.1')
    monkeypatch.setattr('pyright._utils.__version__', '1.0.0')
    monkeypatch.setenv('PYRIGHT_PYTHON_PYLANCE_VERSION', '1.0.0')
    assert not _should_warn_version(args=(), quiet=None)


def test_force_version_env_var(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.1')
    monkeypatch.setattr('pyright._utils.__version__', '1.0.0')
    monkeypatch.setenv('PYRIGHT_PYTHON_FORCE_VERSION', '0.9.9')
    assert _should_warn_version(args=(), quiet=None)


def test_version_comparison(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.1')
    monkeypatch.setattr('pyright._utils.__version__', '1.0.0')
    assert _should_warn_version(args=(), quiet=None)

    monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.0')
    monkeypatch.setattr('pyright._utils.__version__', '1.0.0')
    assert not _should_warn_version(args=(), quiet=None)

    monkeypatch.setattr('pyright._utils.get_latest_version', lambda: None)
    assert not _should_warn_version(args=(), quiet=None)


def test_default_behavior(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr('pyright._utils.get_latest_version', lambda: '1.0.1')
    monkeypatch.setattr('pyright._utils.__version__', '1.0.0')
    assert _should_warn_version(args=(), quiet=None)



import os
import sys
import subprocess
from typing import TYPE_CHECKING, Tuple
from pathlib import Path
from unittest import mock

import pytest
from pytest_subprocess import FakeProcess

import pyright
import pyright.node
from pyright.utils import maybe_decode

if TYPE_CHECKING:
    from _pytest.capture import CaptureFixture


# TODO: test binary resolution


@pytest.mark.parametrize(
    'version,output',
    [
        [(10, 100, 100), '10.100.100'],
        [(7, 1, 4), b'7.1.4'],
        [(1, 0, 0), b'1.0.0'],
        [(7, 6, 7), b'7.6.7'],
        [
            (7, 1, 3),
            b'(node:3388) ExperimentalWarning: The fs.promises API is experimental\n7.1.3',
        ],
    ],
)
def test_target_version(
    node: str,
    fake_process: FakeProcess,
    output: bytes,
    version: Tuple[int, ...],
) -> None:
    fake_process.register_subprocess(  # pyright: ignore[reportUnknownMemberType]
        [node, '--version'], stdout=output
    )
    assert pyright.node.version('node') == version


def test_target_version_not_found(
    node: str,
    fake_process: FakeProcess,
    capsys: 'CaptureFixture[str]',
) -> None:
    fake_process.register_subprocess(  # pyright: ignore[reportUnknownMemberType]
        [node, '--version'], stdout='hello world'
    )

    with pytest.raises(pyright.errors.VersionCheckFailed) as exc:
        pyright.node.version('node')

    captured = capsys.readouterr()
    assert captured.out == ''
    assert captured.err == 'hello world\n'
    assert exc.match('Could not find version from `node --version`, see output above')


def test_run_env_argument(tmp_path: Path) -> None:
    """Ensure the `run()` function can accept an `env` argument."""
    tmp_path.joinpath('test.js').write_text('console.log(process.env.MY_ENV_VAR)')
    proc = pyright.node.run(
        'node',
        'test.js',
        env={**os.environ, 'MY_ENV_VAR': 'hello!'},
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    )
    assert proc.returncode == 0
    assert maybe_decode(proc.stdout) == 'hello!\n'


@mock.patch('pyright.node.NODE_VERSION', '13.1.0')
@mock.patch('pyright.node.USE_GLOBAL_NODE', False)
@mock.patch('pyright.node.USE_NODEJS_WHEEL', False)
def test_node_version_env() -> None:
    """Ensure the custom version is respected."""
    proc = pyright.node.run(
        'node',
        '--version',
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    )
    assert proc.returncode == 0
    assert maybe_decode(proc.stdout).strip() == 'v13.1.0'


@mock.patch('pyright.node.USE_GLOBAL_NODE', False)
@mock.patch('pyright.node.USE_NODEJS_WHEEL', False)
@mock.patch('pyright.node.NODE_VERSION', None)
@mock.patch('pyright.node.BINARIES_DIR', pyright.node.BINARIES_DIR.joinpath('empty'))
def test_nodeenv_flaky_error(fake_process: FakeProcess) -> None:
    """A helpful error is raised when nodeenv fails."""
    fake_process.register_subprocess(  # pyright: ignore[reportUnknownMemberType]
        [sys.executable, '-m', 'nodeenv', str(pyright.node.ENV_DIR)],
        stdout='this thing is flaky',
        returncode=1,
    )

    with pytest.raises(RuntimeError, match=r'install pyright\[nodejs\]'):
        pyright.node.run(
            'node',
            '--version',
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )


def test_update_path_env(tmp_path: Path) -> None:
    """The _update_path_env() function correctly appends the target binary path to the PATH environment variable"""
    target = tmp_path / 'bin'
    target.mkdir()

    sep = os.pathsep

    # known PATH separators - please update if need be
    assert sep in {':', ';'}

    # no env
    path = pyright.node._update_path_env(env=None, target_bin=target)
    assert path.startswith(f'{target.absolute()}{sep}')

    # env without PATH
    path = pyright.node._update_path_env(
        env={'FOO': 'bar'},
        target_bin=target,
    )
    assert path.startswith(f'{target.absolute()}{sep}')

    # env with empty PATH
    path = pyright.node._update_path_env(
        env={'PATH': ''},
        target_bin=target,
    )
    assert path.startswith(f'{target.absolute()}{sep}')

    # env with set PATH without the separator postfix
    path = pyright.node._update_path_env(
        env={'PATH': '/foo'},
        target_bin=target,
    )
    assert path == f'{target.absolute()}{sep}/foo'

    # env with set PATH with the separator as a prefix
    path = pyright.node._update_path_env(
        env={'PATH': f'{sep}/foo'},
        target_bin=target,
    )
    assert path == f'{target.absolute()}{sep}/foo'

    # returned env included non PATH environment variables
    path = pyright.node._update_path_env(
        env={'PATH': '/foo', 'FOO': 'bar'},
        target_bin=target,
    )
    assert path == f'{target.absolute()}{sep}/foo'

    # accepts a custom path separator
    path = pyright.node._update_path_env(
        env={'PATH': '/foo'},
        target_bin=target,
        sep='---',
    )
    assert path == f'{target.absolute()}---/foo'



import pytest

from pyright.types import check_target


def test_check_target() -> None:
    with pytest.raises(TypeError) as exc:
        check_target('foo')

    assert exc.match(r'foo is not a valid target, expected one of ((npm|node), )+')
    check_target('npm')
    check_target('node')



from __future__ import annotations

import os
import re
import sys
import json
import platform
import subprocess
from typing import TYPE_CHECKING
from pathlib import Path

from packaging import version

import pyright
from pyright import __pyright_version__
from tests.utils import assert_matches
from pyright.utils import maybe_decode

if TYPE_CHECKING:
    from _pytest.monkeypatch import MonkeyPatch


VERSION_REGEX = re.compile(r'pyright (?P<version>\d+\.\d+\.\d+)')


def test_module_invocation() -> None:
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
    )
    assert proc.returncode == 0
    match = assert_matches(VERSION_REGEX, proc.stdout.decode('utf-8'))
    assert match.group(1) == __pyright_version__


def test_module_invocation_version() -> None:
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_FORCE_VERSION='1.1.223'),
    )
    assert proc.returncode == 0
    match = assert_matches(VERSION_REGEX, proc.stdout.decode('utf-8'))
    assert match.group(1) == '1.1.223'


def test_module_invocation_latest_version() -> None:
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_FORCE_VERSION='latest'),
    )
    assert proc.returncode == 0
    match = assert_matches(VERSION_REGEX, proc.stdout.decode('utf-8'))
    assert version.parse(match.group(1)) >= version.parse(__pyright_version__)


def test_module_invocation_pylance_version() -> None:
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_PYLANCE_VERSION='2023.11.11'),
    )
    assert proc.returncode == 0
    match = assert_matches(VERSION_REGEX, proc.stdout.decode('utf-8'))
    assert match.group(1) == '1.1.334'


def test_module_invocation_pylance_version_latest_prerelease() -> None:
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_PYLANCE_VERSION='latest-prerelease'),
    )
    assert proc.returncode == 0
    match = assert_matches(VERSION_REGEX, proc.stdout.decode('utf-8'))
    # Can't predict which version of pyright is currently used by pylance. Just
    # ensure that the process ran succesfully and some pyright version appeared
    # in the output.
    assert len(match.groups()) == 1


def test_entry_point() -> None:
    proc = subprocess.run(
        ['pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    match = assert_matches(VERSION_REGEX, output)
    assert match.group(1) == __pyright_version__


def test_long_arguments(tmp_path: Path) -> None:
    """Long arguments (e.g. --outputjson) are passed to the pyright CLI"""
    tmp_path.joinpath('foo.py').write_text('reveal_type(1)')

    result = pyright.run('--outputjson', 'foo.py', stdout=subprocess.PIPE)
    assert result.returncode == 0

    data = json.loads(result.stdout)
    assert data['generalDiagnostics'][0]['message'] == 'Type of "1" is "Literal[1]"'


def test_argument_separator(tmp_path: Path) -> None:
    """Ensure the npx / pyright argument separator correctly separates arguments."""
    tmp_path.joinpath('foo.py').write_text('reveal_type(1)')

    result = pyright.run('foo.py', stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    assert result.returncode == 0

    output = maybe_decode(result.stdout)
    assert 'does not exist' not in output


def test_explicit_version_new_version_warning() -> None:
    """A new version is available warning is emitted explicitly using an older version"""
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_FORCE_VERSION='1.1.222'),
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    assert 'WARNING: there is a new pyright version available' in output


def test_explicit_latest_no_new_version_warning() -> None:
    """No new version warning is emitted when explicitly setting to `latest`"""
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_FORCE_VERSION='latest'),
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    assert 'WARNING: there is a new pyright version available' not in output


def test_pylance_version_no_new_version_warning() -> None:
    """No new version warning is emitted when PYRIGHT_PYTHON_PYLANCE_VERSION is set"""
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_PYLANCE_VERSION='2023.11.10'),
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    assert 'WARNING: there is a new pyright version available' not in output


def test_output_json_no_warning() -> None:
    """If the --outputjson flag is set then no warning is emitted"""
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version', '--outputjson'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(os.environ, PYRIGHT_PYTHON_FORCE_VERSION='1.1.222'),
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    assert 'WARNING: there is a new pyright version available' not in output


def test_ignore_warnings_config_no_warning() -> None:
    """If the --outputjson flag is set then no warning is emitted"""
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(
            os.environ,
            PYRIGHT_PYTHON_FORCE_VERSION='1.1.222',
            PYRIGHT_PYTHON_IGNORE_WARNINGS='1',
        ),
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    assert 'WARNING: there is a new pyright version available' not in output


def test_nodeenv() -> None:
    """Ensure nodeenv is successfully downloaded and used"""
    subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
        stdout=subprocess.PIPE,
        env=dict(
            os.environ,
            PYRIGHT_PYTHON_GLOBAL_NODE='0',
        ),
    )


def test_user_special_characters() -> None:
    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        env={
            **os.environ,
            'LOGNAME': 'alice@example.com',
        },
    )
    assert proc.returncode == 0
    output = proc.stdout.decode('utf-8')
    assert str(__pyright_version__) in output


def test_package_json_in_parent_dir(tmp_path: Path, monkeypatch: MonkeyPatch) -> None:
    """The CLI can be installed successfully when there is a `package.json` file
    in a parent directory.
    """
    if platform.system() == 'Windows':
        # hack to avoid WinError 206
        tmp_path = tmp_path.parent.parent / 'abc'

    tmp_path.mkdir(exist_ok=True, parents=True)
    tmp_path.joinpath('package.json').write_text('{"name": "another package.json"}')

    cache_dir = tmp_path / 'foo' / 'bar'
    cache_dir.mkdir(exist_ok=True, parents=True)

    monkeypatch.setenv('PYRIGHT_PYTHON_CACHE_DIR', str(cache_dir))

    proc = subprocess.run(
        [sys.executable, '-m', 'pyright', '--version'],
        check=True,
    )
    assert proc.returncode == 0



from __future__ import annotations

import os
import re
import sys
import json
import shutil
import logging
import platform
import subprocess
import importlib.util
from typing import Any, Dict, Tuple, Union, Mapping, Optional, NamedTuple, cast
from pathlib import Path
from functools import lru_cache
from typing_extensions import Literal, assert_never

from . import errors
from .types import Target, check_target
from .utils import env_to_bool, get_bin_dir, get_env_dir, maybe_decode

log: logging.Logger = logging.getLogger(__name__)

ENV_DIR: Path = get_env_dir()
BINARIES_DIR: Path = get_bin_dir(env_dir=ENV_DIR)
USE_GLOBAL_NODE = env_to_bool('PYRIGHT_PYTHON_GLOBAL_NODE', default=True)
USE_NODEJS_WHEEL = env_to_bool('PYRIGHT_PYTHON_NODEJS_WHEEL', default=True)
NODE_VERSION = os.environ.get('PYRIGHT_PYTHON_NODE_VERSION', default=None)
VERSION_RE = re.compile(r'\d+\.\d+\.\d+')


def _is_windows() -> bool:
    return platform.system().lower() == 'windows'


def _postfix_for_target(target: Target) -> str:
    if not _is_windows():
        return ''

    if target == 'node':
        return '.exe'
    return '.cmd'


def _ensure_node_env(target: Target) -> Path:
    log.debug('Checking for nodeenv %s binary', target)

    path = _get_nodeenv_path(target)
    log.debug('Using %s path for binary', path)

    if path.exists() and not NODE_VERSION:
        log.debug('Binary at %s exists, skipping nodeenv installation', path)
    else:
        log.debug('Installing nodeenv as a binary at %s could not be found', path)
        _install_node_env()

    if not path.exists():
        raise errors.BinaryNotFound(path=path, target=target)
    return path


def _get_nodeenv_path(target: Target) -> Path:
    return BINARIES_DIR.joinpath(target + _postfix_for_target(target))


def _get_global_binary(target: Target) -> Optional[Path]:
    log.debug('Checking for global target binary: %s', target)

    path = target + _postfix_for_target(target)

    which = shutil.which(path)
    if which is not None:
        log.debug('Found global binary at: %s', which)

        path = Path(which)
        if path.exists():
            log.debug('Global binary exists at: %s', which)
            return path

    log.debug('Global target binary: %s not found', target)
    return None


def _install_node_env() -> None:
    log.debug('Installing nodeenv to %s', ENV_DIR)
    args = [sys.executable, '-m', 'nodeenv']
    if NODE_VERSION:
        log.debug(f'Using user specified node version: {NODE_VERSION}')
        args += ['--node', NODE_VERSION, '--force']
    args.append(str(ENV_DIR))
    log.debug('Running command with args: %s', args)

    try:
        subprocess.run(args, check=True)
    except subprocess.CalledProcessError as exc:
        raise RuntimeError(
            'nodeenv failed; for more reliable node.js binaries try `pip install pyright[nodejs]`'
        ) from exc


class GlobalStrategy(NamedTuple):
    type: Literal['global']
    path: Path


class NodeJSWheelStrategy(NamedTuple):
    type: Literal['nodejs_wheel']


class NodeenvStrategy(NamedTuple):
    type: Literal['nodeenv']
    path: Path


Strategy = Union[GlobalStrategy, NodeJSWheelStrategy, NodeenvStrategy]


def _resolve_strategy(target: Target) -> Strategy:
    if USE_NODEJS_WHEEL:
        if importlib.util.find_spec('nodejs_wheel') is not None:
            log.debug('Using nodejs_wheel package for resolving binaries')
            return NodeJSWheelStrategy(type='nodejs_wheel')

    if USE_GLOBAL_NODE:
        path = _get_global_binary(target)
        if path is not None:
            log.debug('Using global %s binary', target)
            return GlobalStrategy(type='global', path=path)

    log.debug('Installing binaries using nodeenv')
    return NodeenvStrategy(type='nodeenv', path=_ensure_node_env(target))


def run(
    target: Target, *args: str, **kwargs: Any
) -> Union['subprocess.CompletedProcess[bytes]', 'subprocess.CompletedProcess[str]']:
    check_target(target)

    strategy = _resolve_strategy(target)
    if strategy.type == 'global':
        node_args = [str(strategy.path), *args]
        log.debug('Running global node command with args: %s', node_args)
        return cast(
            'subprocess.CompletedProcess[str] | subprocess.CompletedProcess[bytes]',
            subprocess.run(node_args, **kwargs),
        )
    elif strategy.type == 'nodejs_wheel':
        import nodejs_wheel

        if target == 'node':
            return cast(
                'subprocess.CompletedProcess[str] | subprocess.CompletedProcess[bytes]',
                nodejs_wheel.node(args, return_completed_process=True, **kwargs),
            )
        elif target == 'npm':
            return cast(
                'subprocess.CompletedProcess[str] | subprocess.CompletedProcess[bytes]',
                nodejs_wheel.npm(args, return_completed_process=True, **kwargs),
            )
        else:
            assert_never(target)
    elif strategy.type == 'nodeenv':
        env = kwargs.pop('env', None) or os.environ.copy()
        env.update(get_env_variables())

        # If we're using `nodeenv` to resolve the node binary then we also need
        # to ensure that `node` is in the PATH so that any install scripts that
        # assume it is present will work.
        env.update(PATH=_update_path_env(env=env, target_bin=strategy.path.parent))
        node_args = [str(strategy.path), *args]
        log.debug('Running nodeenv command with args: %s', node_args)
        return cast(
            'subprocess.CompletedProcess[str] | subprocess.CompletedProcess[bytes]',
            subprocess.run(node_args, env=env, **kwargs),
        )
    else:
        assert_never(strategy)


def version(target: Target) -> Tuple[int, ...]:
    proc = run(target, '--version', stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    output = maybe_decode(proc.stdout)
    match = VERSION_RE.search(output)
    if not match:
        print(output, file=sys.stderr)
        raise errors.VersionCheckFailed(f'Could not find version from `{target} --version`, see output above')

    info = tuple(int(value) for value in match.group(0).split('.'))
    log.debug('Version check for %s returning %s', target, info)
    return info


@lru_cache(maxsize=None)
def latest(package: str) -> str:
    """Return the latest version for the given package"""
    proc = run(
        'npm',
        'info',
        package,
        'version',
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    )
    stdout = maybe_decode(proc.stdout)

    if proc.returncode != 0:
        print(stdout, file=sys.stderr)
        raise errors.VersionCheckFailed(f'Version check for {package} failed, see output above.')

    match = VERSION_RE.search(stdout)
    if not match:
        print(stdout, file=sys.stderr)
        raise errors.VersionCheckFailed(f'Could not find version for {package}, see output above')

    value = match.group(0)
    log.debug('Version check for %s returning %s', package, value)
    return value


def get_env_variables() -> Dict[str, Any]:
    """Return the environmental variables that should be passed to a binary"""
    # NOTE: I do not actually know if these result in the intended behaviour
    #       I simply copied them from bin/shim in nodeenv
    return {
        'NODE_PATH': str(ENV_DIR / 'lib' / 'node_modules'),
        'NPM_CONFIG_PREFIX': str(ENV_DIR),
        'npm_config_prefix': str(ENV_DIR),
    }


def get_pkg_version(pkg: Path) -> str | None:
    """Given a path to a `package.json` file, parse it and returns the `version` property

    Returns `None` if the version could not be resolved for any reason.
    """
    if not pkg.exists():
        return None

    try:
        data = json.loads(pkg.read_text())
    except Exception:
        # TODO: test this
        log.debug('Ignoring error while reading/parsing the %s file', pkg, exc_info=True)
        return None

    return data.get('version')


def _update_path_env(
    *,
    env: Mapping[str, str] | None,
    target_bin: Path,
    sep: str = os.pathsep,
) -> str:
    """Returns a modified version of the `PATH` environment variable that has been updated
    to include the location of the downloaded Node binaries.
    """
    if env is None:
        env = dict(os.environ)

    log.debug('Attempting to prepend %s to the PATH', target_bin)
    assert target_bin.exists(), f'Target directory {target_bin} does not exist'

    path = env.get('PATH', '') or os.environ.get('PATH', '')
    if path:
        log.debug('Found PATH contents: %s', path)

        # handle the case where the PATH already starts with the separator (this probably shouldn't happen)
        if path.startswith(sep):
            path = f'{target_bin.absolute()}{path}'
        else:
            path = f'{target_bin.absolute()}{sep}{path}'
    else:
        # handle the case where there is no PATH set (unlikely / impossible to actually happen?)
        path = str(target_bin.absolute())

    log.debug('Using PATH environment variable: %s', path)
    return path
